﻿' -----------------------------------------------------------------------------------------
'
' Rob91FF.vb                                                        Ulrich Müller  02.04.08
'                                                                   PCS1700 VB 2005 Express
' Demo Robot aus Profi Computing 1991 : SingleThread-Lösung
'      Die Ansteuerung der Home Position erfolgt hier nur bei Start des Programms
'      Der simultane Ablauf wurde gestrichen, da die Tonne dann nicht sicher gegriffen
'      werden kann.
'      Achtung : M1 (I1 = Ende, I2 = Impuls), M2 (I3, I4), M3 ( I5, I6)
'                Die Drehrichtung der Motoren wurde umgekehrt
'      Basis ist die Assembly FishFace2005.DLL
'
' -----------------------------------------------------------------------------------------

Option Explicit On
Option Strict Off
Imports FishFace40

Public Class Rob91FF

  Dim ft As New FishFace()

  Const mDreh = Out.M1
  Const cDrehRechts = Dir.Links
  Const cDrehLinks = Dir.Rechts
  Const eDrehEnd = Inp.I1
  Const eDrehImpuls = Inp.I2
  Dim DrehPos1 As Integer = 112
  Dim DrehPos2 As Integer = 20
  Dim DrehAktPos As Integer = 0

  Const mArm = Out.M2
  Const cArmAuf = Dir.Links
  Const cArmAb = Dir.Rechts
  Const eArmOben = Inp.I3
  Const eArmImpuls = Inp.I4
  Const cArmOben = 0
  Dim ArmUnten As Integer = 15

  Const mZange = Out.M3
  Const cZangeAuf = Dir.Links
  Const cZangeZu = Dir.Rechts
  Const eZangeOffen = Inp.I5
  Const eZangeImpuls = Inp.I6
  Const cZangeOffen = 0
  Dim ZangeZu As Integer = 26

  Sub DrehHome()
    lblDreh.Text = "Action"
    ft.SetMotor(mDreh, cDrehRechts)
    ft.WaitForInput(eDrehEnd)
    ft.SetMotor(mDreh, Dir.Aus)
    lblDreh.Text = "Home"
    DrehAktPos = 0
  End Sub

  Sub Drehen(ByVal Pos As Integer)
    lblDreh.Text = "Action"
    ft.SetMotor(mDreh, IIf(Pos < DrehAktPos, cDrehRechts, cDrehLinks))
    ft.WaitForChange(eDrehImpuls, Math.Abs(DrehAktPos - Pos))
    ft.SetMotor(mDreh, Dir.Aus)
    DrehAktPos = Pos
    lblDreh.Text = "Pos : " & Pos.ToString()
  End Sub

  Sub Arm(ByVal Pos As Integer)
    lblArm.Text = "Action"
    If Pos = 0 Then
      ft.SetMotor(mArm, cArmAuf)
      ft.WaitForInput(eArmOben)
    Else
      ft.SetMotor(mArm, cArmAb)
      ft.WaitForChange(eArmImpuls, Pos)
    End If
    ft.SetMotor(mArm, Dir.Aus)
    lblArm.Text = IIf(Pos = 0, "Oben", "Unten")
  End Sub

  Sub Zange(ByVal Pos As Integer)
    lblZange.Text = "Action"
    If Pos = 0 Then
      ft.SetMotor(mZange, cZangeAuf)
      ft.WaitForInput(eZangeOffen)
    Else
      ft.SetMotor(mZange, cZangeZu)
      ft.WaitForChange(eZangeImpuls, Pos)
    End If
    ft.SetMotor(mZange, Dir.Aus)
    lblZange.Text = IIf(Pos = 0, "Offen", "Geschlossen")
  End Sub

  Private Sub Action()
    ZangeZu = numZange.Value
    ArmUnten = numArm.Value
    DrehPos1 = numPos1.Value
    DrehPos2 = numPos2.Value

    lblStatus.Text = "Fährt auf Grundstellung"
    Zange(cZangeOffen)
    Arm(cArmOben)
    DrehHome()
    Do
      lblStatus.Text = "Zu Position 1"
      Drehen(DrehPos1)
      Arm(ArmUnten)
      Zange(ZangeZu)
      Arm(cArmOben)

      lblStatus.Text = "Zu Position 2"
      Drehen(DrehPos2)
      Arm(ArmUnten)
      Zange(cZangeOffen)
      Arm(cArmOben)
    Loop Until ft.Finish()
  End Sub

#Region "--- ProgramControl ---"
  Private Sub cmdAction_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) _
  Handles cmdAction.Click
    ' --- Open erste ROBO Interface an USB oder Intelligent Interface an COM ---
    '     während des Ablaufs von Action gesperrt
    Try
      If cboPortName.SelectedIndex = 0 Then
        ft.OpenInterface(IFTypen.ftROBO_first_USB, 0)
      Else
        ft.OpenInterface(IFTypen.ftIntelligent_IF, cboPortName.SelectedIndex, 5)
      End If
      cmdAction.Enabled = False
      cmdEnde.Text = "&HALT"
      lblStatus.Text = "--- Bei der Arbeit ---"

      Action()          ' --- Nutz-Routine für die Anwendung

      ft.ClearMotors()
    Catch ex As FishFaceException
      MsgBox(ex.Message)
    Finally
      ft.CloseInterface()
      cmdAction.Enabled = True
      cmdEnde.Text = "&ENDE"
      lblStatus.Text = "--- Auf ein Neues ---"
      cmdEnde.Focus()
    End Try

  End Sub

  Private Sub cmdEnde_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdEnde.Click
    ' --- Programmende nur, wenn Action nicht läuft ---
    If cmdEnde.Text = "&HALT" Then ft.NotHalt = True Else Me.Close()
  End Sub

  Private Sub ManBeispiele_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
    ' --- Default erstes ROBO Interface an USB ---
    cboPortName.SelectedIndex = 2
  End Sub

  Private Sub ManBeispiele_FormClosing(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing
    ' --- Wenn Action läuft, wird ein Programmabbruch über (x) unterbunden ---
    If cmdEnde.Text = "&HALT" Then e.Cancel = True
  End Sub

#End Region


End Class
