﻿' -----------------------------------------------------------------------------------------
'
' LLmain.vb                                                         Ulrich Müller  03.04.08
'                                                                   PCS1700 VB 2005 Express
' Demo Robot aus Profi Computing 1991 : MultiThread-Lösung
'   Drehkranz, Arm und Zange laufen in eigenen Threads und synchronisieren sich
'   über ManualResetEvents. 
'
'   Ablauf entsprechend Lucky Logic Programm (Endlosschleife bis ESC-Taste) :
'   1. Anfahren Grundstellung
'      simultan : Dreh auf Endtaster, Arm hoch, Zange öffnen
'   2. Anfahren Position 1
'      simultan : Dreh auf Pos 1, Arm nach unten
'      danach   : Zange schließen
'   3. Anfahren Position 2
'      Dreh auf Pos 2 (Arm bleibt unten)
'
'  Achtung : M1 (I1 = Ende, I2 = Impuls), M2 (I3, I4), M3 ( I5, I6)
'            Die Drehrichtung der Motoren wurde umgekehrt
'  Basis ist die Assembly FishFace2005.DLL
'
' -----------------------------------------------------------------------------------------

Option Explicit On
Option Strict Off
Imports FishFace40
Imports System.Threading

Public Class Rob91LL

  Dim ft As New FishFace()
  Dim td, ta, tz As Thread
  Dim tdEnd, tdPos1, tdPos2, tzAuf, tzZu As ManualResetEvent

  Const mDreh = Out.M1
  Const cDrehRechts = Dir.Links
  Const cDrehLinks = Dir.Rechts
  Const eDrehEnd = Inp.I1
  Const eDrehImpuls = Inp.I2
  Dim DrehPos1 As Integer = 112
  Dim DrehPos2 As Integer = 20
  Dim DrehAktPos As Integer = 0

  Const mArm = Out.M2
  Const cArmAuf = Dir.Links
  Const cArmAb = Dir.Rechts
  Const eArmOben = Inp.I3
  Const eArmImpuls = Inp.I4
  Const cArmOben = 0
  Dim ArmUnten As Integer = 15

  Const mZange = Out.M3
  Const cZangeAuf = Dir.Links
  Const cZangeZu = Dir.Rechts
  Const eZangeOffen = Inp.I5
  Const eZangeImpuls = Inp.I6
  Const cZangeOffen = 0
  Dim ZangeZu As Integer = 26

  
  Private Sub Action()
    ' --- Anlegen und Starten der Threads ---
    td = New Thread(New ThreadStart(AddressOf tdExecute))
    td.IsBackground = True
    tdEnd = New ManualResetEvent(False)
    tdPos1 = New ManualResetEvent(False)
    tdPos2 = New ManualResetEvent(False)
    td.Start()

    ta = New Thread(New ThreadStart(AddressOf taExecute))
    ta.IsBackground = True
    ta.Start()

    tz = New Thread(New ThreadStart(AddressOf tzExecute))
    tz.IsBackground = True
    tzAuf = New ManualResetEvent(False)
    tzZu = New ManualResetEvent(False)
    tz.Start()

    ' --- Aufrechterhalten des HauptThreads ---
    Do
      ft.Pause(123)
    Loop Until ft.Finish()
    td.Abort()
    ta.Abort()
    tz.Abort()
  End Sub

  ' --- ThreadRoutine für den Drehkranz ---
  Sub tdExecute()
    Do
      ' --- Grundstellung ---
      lblStatus.Text = "Anfahren Grundstellung"
      tdPos2.Reset()
      lblDreh.Text = "Action"
      ft.SetMotor(mDreh, cDrehRechts)
      ft.WaitForInput(eDrehEnd)
      ft.SetMotor(mDreh, Dir.Aus)
      lblDreh.Text = "EndTaster"
      tdEnd.Set()

      ' --- Anfahren Position 1 ---
      lblStatus.Text = "Anfahren Position 1"
      tdEnd.Reset()
      lblDreh.Text = "Action"
      ft.SetMotor(mDreh, cDrehLinks)
      ft.WaitForChange(eDrehImpuls, DrehPos1)
      ft.SetMotor(mDreh, Dir.Aus)
      lblDreh.Text = "Position 1"
      tdPos1.Set()
      tzZu.WaitOne()

      ' --- Anfahren Position 2 ---
      lblStatus.Text = "Anfahren Position 2"
      tdPos1.Reset()
      lblDreh.Text = "Action"
      ft.SetMotor(mDreh, cDrehRechts)
      ft.WaitForChange(eDrehImpuls, DrehPos2)
      ft.SetMotor(mDreh, Dir.Aus)
      lblDreh.Text = "Position 2"
      tdPos2.Set()
      tzAuf.WaitOne()
    Loop Until ft.Finish()
  End Sub

  ' --- ThreadRoutine des Arms ---
  Sub taExecute()
    Do
      ' --- Arm aufwärts ---
      lblArm.Text = "Action"
      ft.SetMotor(mArm, cArmAuf)
      ft.WaitForInput(eArmOben)
      ft.SetMotor(mArm, Dir.Aus)
      lblArm.Text = "Oben"
      tdEnd.WaitOne()

      ' --- Arm abwärts ---
      lblArm.Text = "Action"
      ft.SetMotor(mArm, cArmAb)
      ft.WaitForChange(eArmImpuls, ArmUnten)
      ft.SetMotor(mArm, Dir.Aus)
      lblArm.Text = "Unten"
      tdPos1.WaitOne()
      tdPos2.WaitOne()
    Loop Until ft.Finish()
  End Sub

  ' --- ThreadRoutine der Zange ---
  Sub tzExecute()
    Do
      ' --- Zange öffnen ---
      lblZange.Text = "Action"
      tzAuf.Reset()
      ft.SetMotor(mZange, cZangeAuf)
      ft.WaitForInput(eZangeOffen)
      ft.SetMotor(mZange, Dir.Aus)
      lblZange.Text = "Offen"
      tzAuf.Set()
      tdPos1.WaitOne()

      ' --- Zange schließen ---
      lblZange.Text = "Action"
      tzZu.Reset()
      ft.SetMotor(mZange, cZangeZu)
      ft.WaitForChange(eZangeImpuls, ZangeZu)
      ft.SetMotor(mZange, Dir.Aus)
      lblZange.Text = "Geschlossen"
      tzZu.Set()
      tdPos2.WaitOne()
    Loop Until ft.Finish()
  End Sub

#Region "--- ProgramControl ---"
  Private Sub cmdAction_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) _
  Handles cmdAction.Click
    ' --- Open erste ROBO Interface an USB oder Intelligent Interface an COM ---
    '     während des Ablaufs von Action gesperrt
    Try
      If cboPortName.SelectedIndex = 0 Then
        ft.OpenInterface(IFTypen.ftROBO_first_USB, 0)
      Else
        ft.OpenInterface(IFTypen.ftIntelligent_IF, cboPortName.SelectedIndex, 5)
      End If
      cmdAction.Enabled = False
      cmdEnde.Text = "&HALT"
      lblStatus.Text = "--- Bei der Arbeit ---"

      Action()          ' --- Nutz-Routine für die Anwendung

      ft.ClearMotors()
    Catch ex As FishFaceException
      MsgBox(ex.Message)
    Finally
      ft.CloseInterface()
      cmdAction.Enabled = True
      cmdEnde.Text = "&ENDE"
      lblStatus.Text = "--- Auf ein Neues ---"
      cmdEnde.Focus()
    End Try

  End Sub

  Private Sub cmdEnde_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdEnde.Click
    ' --- Programmende nur, wenn Action nicht läuft ---
    If cmdEnde.Text = "&HALT" Then ft.NotHalt = True Else Me.Close()
  End Sub

  Private Sub ManBeispiele_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
    ' --- Default erstes ROBO Interface an USB ---
    cboPortName.SelectedIndex = 2
    Label.CheckForIllegalCrossThreadCalls = False
  End Sub

  Private Sub ManBeispiele_FormClosing(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing
    ' --- Wenn Action läuft, wird ein Programmabbruch über (x) unterbunden ---
    If cmdEnde.Text = "&HALT" Then e.Cancel = True
  End Sub

#End Region


End Class
